#version 330 compatibility


// Inputs
in vec2 mc_Entity;
in vec4 at_midBlock;

// Outputs
out vec2 texCoord;
out vec2 lightCoord;
out vec4 vertColor;
out float camVertDist;
out float vertDist;
out vec3 viewNormal;

// Uniforms
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform vec3 cameraPosition;
uniform vec3 eyePosition;
uniform sampler2D noisetex;
uniform float frameTimeCounter;
uniform bool is_on_ground;


float getHeight(vec2 position, vec3 sio, bool flatBottom) {
    float shift = texture(noisetex, position/1024+vec2(frameTimeCounter/(256/sio.x))).r*sio.y*0.55+sio.z;
    // If flatBottom is true and the vertex is connected to the bottom face then don't shift up
    if (flatBottom && at_midBlock.y == 32) shift = min(0., shift);
    // Return the shift
    return shift;
}

vec3 normalFromHeight(vec2 pos, vec3 sio, bool flatBottom, bool flipNormal) {
    float stepSize = .5;
    vec2 e = vec2(stepSize, 0);
    vec3 px1 = vec3(pos.x - e.x, getHeight(pos - e.xy, sio, flatBottom), pos.y - e.y);
    vec3 px2 = vec3(pos.x + e.x, getHeight(pos + e.xy, sio, flatBottom), pos.y + e.y);
    vec3 py1 = vec3(pos.x - e.y, getHeight(pos - e.yx, sio, flatBottom), pos.y - e.x);
    vec3 py2 = vec3(pos.x + e.y, getHeight(pos + e.yx, sio, flatBottom), pos.y + e.x);
    if (flipNormal) return normalize(cross(py2 - py1, px2 - px1));
    return normalize(cross(px2 - px1, py2 - py1));
}

void main() {
    gl_Position = ftransform();
    texCoord = gl_MultiTexCoord0.xy;
    lightCoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    vertColor = gl_Color;
    camVertDist = length((gl_ModelViewMatrix * gl_Vertex).xyz);
    vertDist = camVertDist - length(eyePosition - cameraPosition);
    viewNormal = gl_NormalMatrix * gl_Normal.xyz;

    // Vertex position
    vec3 eyeCameraPosition = cameraPosition + gbufferModelViewInverse[3].xyz;
    vec3 worldVertex = mat3(gbufferModelViewInverse)*(gl_ModelViewMatrix*gl_Vertex).xyz+eyeCameraPosition;

    // Only apply waves if the id is 1 (wavy stuff) do some magic to keep the upper face from clipping through the block below
    if (mc_Entity.x >= 2) {
        // Speed, Intensity, Offset
        vec3 sio = vec3(1., 1., 0.);
        bool flatBottom = false;
        // Set sio for various block groups
        if (mc_Entity.x == 2) {sio = vec3(1., 1., -.4); flatBottom = true;}
        if (mc_Entity.x == 3) {sio = vec3(1.2, 0.6, -.15); flatBottom = false;}

        float vertShift = getHeight(worldVertex.xz, sio, flatBottom);
        // We check whether the new position clips out of bounds (down only)
        // Subtracting 0.001 to avoid limiting the bottom face by moving the bounds down
        if (floor(worldVertex.y-0.001) > floor(worldVertex.y-0.001 + vertShift)) worldVertex.y = floor(worldVertex.y);
        else worldVertex.y += vertShift;

        //if (at_midBlock.y == 32) worldVertex.y += 1; IRIS BUG at_midBlock changes when other blocks are placed near
        // Move it a bit up to avoid z fighting (common with thin flowing water)
        worldVertex.y = floor(worldVertex.y*16)/16+0.001;

        // New normal
        vec3 originalNormal = mat3(gbufferModelViewInverse) * viewNormal;
        if (originalNormal.y > 0.1 && originalNormal.y < -0.1) {
            bool flipNormal = false;
            if (originalNormal.y > 0.1) flipNormal = true;
            viewNormal = gl_NormalMatrix * normalFromHeight(worldVertex.xz, sio, flatBottom, flipNormal);
        }
    }
    // Tall grass and other crossed planes should just take the normal of the face below them (up)
    if (mc_Entity.x == 1) viewNormal = gl_NormalMatrix * vec3(0, 1, 0);

    // To clip space
    gl_Position = gbufferProjection * vec4(mat3(gbufferModelView)*(worldVertex-eyeCameraPosition), 1);
}
